/* ----------------------------------------------------------------------------
 *     PROJECT : EURES
 *
 *     PACKAGE : com.jpattern.orm.annotation
 *        FILE : CascadeType.java
 *
 *  CREATED BY : ARHS Developments
 *          ON : Mar 10, 2013
 *
 * MODIFIED BY : ARHS Developments
 *          ON : $LastChangedDate
 *     VERSION : $LastChangedRevision
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2011 European Commission - DG EMPL
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.annotation.cascade;

import com.jpattern.orm.annotation.Cascade;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 10, 2013
 *
 * @author ARHS Developments - Francesco Cina
 * @version $Revision
 */
public enum CascadeType {

    /**
     * Cascade delete, save and update actions. This is the default behavior if the
     * {@link Cascade} annotation is not specified.
     */
    ALWAYS(       new CascadeInfoImpl(true, true, true)),
    /**
     * Cascade only delete actions.
     */
    DELETE(       new CascadeInfoImpl(true, false, false)),
    /**
     * Cascade delete and save actions.
     */
    DELETE_SAVE(  new CascadeInfoImpl(true, true, false)),
    /**
     * Cascade delete and update actions.
     */
    DELETE_UPDATE(new CascadeInfoImpl(true, false, true)),
    /**
     * Cascade only save actions.
     */
    SAVE(         new CascadeInfoImpl(false, true, false)),
    /**
     * Cascade save and update actions.
     */
    SAVE_UPDATE(  new CascadeInfoImpl(false, true, true)),
    /**
     * Cascade only update actions.
     */
    UPDATE(       new CascadeInfoImpl(false, false, true)),
    /**
     * Cascade is disabled.
     */
    NEVER(        new CascadeInfoImpl(false, false, false));

    private final CascadeInfo info;

    CascadeType(final CascadeInfo info) {
        this.info = info;
    }

    public CascadeInfo getInfo() {
        return info;
    }

}
