package com.jpattern.orm.mapper;

import com.jpattern.orm.cache.CacheManager;
import com.jpattern.orm.cache.SimpleCacheManager;
import com.jpattern.orm.dialect.DBProfile;
import com.jpattern.orm.dialect.UnknownDBProfile;
import com.jpattern.orm.persistor.type.TypeFactory;
import com.jpattern.orm.query.find.cache.CacheStrategy;
import com.jpattern.orm.query.find.cache.CacheStrategyImpl;
import com.jpattern.orm.query.namesolver.PropertiesFactory;
import com.jpattern.orm.validator.NullValidatorService;
import com.jpattern.orm.validator.ValidatorService;


/**
 * 
 * @author Francesco Cina
 *
 * 22/mag/2011
 */
public class NullServiceCatalog implements ServiceCatalog {

    @Override
    public boolean containsTool(final Class<?> clazz) {
        return false;
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    public <T> OrmClassTool<T> getOrmClassTool(final Class<T> clazz) {
        return new NullOrmClassTool();
    }

    @Override
    public TypeFactory getTypeFactory() {
        return new TypeFactory();
    }

    @Override
    public DBProfile getDbProfile() {
        return new UnknownDBProfile();
    }

    @Override
    public ValidatorService getValidatorService() {
        return new NullValidatorService();
    }

    @Override
    public CacheManager getCacheManager() {
        return new SimpleCacheManager();
    }

    @Override
    public PropertiesFactory getPropertiesFactory() {
        return new PropertiesFactory();
    }

    @Override
    public CacheStrategy getCacheStrategy() {
        return new CacheStrategyImpl(this);
    }

}
