package com.jpattern.orm.mapper;

import java.util.Map;
import java.util.HashMap;

import com.jpattern.orm.JPO;
import com.jpattern.orm.cache.CacheManager;
import com.jpattern.orm.cache.SimpleCacheManager;
import com.jpattern.orm.dialect.DBProfile;
import com.jpattern.orm.dialect.UnknownDBProfile;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.persistor.type.TypeFactory;
import com.jpattern.orm.query.find.cache.CacheStrategy;
import com.jpattern.orm.query.find.cache.CacheStrategyImpl;
import com.jpattern.orm.query.namesolver.PropertiesFactory;
import com.jpattern.orm.validator.NullValidatorService;
import com.jpattern.orm.validator.ValidatorService;


/**
 * 
 * @author Francesco Cina
 *
 * 22/mag/2011
 */
public class ServiceCatalogImpl implements ServiceCatalog {

    private final TypeFactory typeFactory = new TypeFactory();
    private final Map<Class<?>, OrmClassTool<?>> classToolMap = new HashMap<Class<?>, OrmClassTool<?>>();
    private final JPO jpOrm;
    private DBProfile dbProfile = new UnknownDBProfile();
    private ValidatorService validatorService = new NullValidatorService();
    private CacheManager cacheManager = new SimpleCacheManager();
    private final PropertiesFactory propertiesFactory = new PropertiesFactory();
    private final CacheStrategy cacheStrategy = new CacheStrategyImpl(this);

    public ServiceCatalogImpl(final JPO jpOrm) {
        this.jpOrm = jpOrm;
    }

    public <T> void put(final Class<T> clazz, final OrmClassTool<T> ormClassTool) {
        classToolMap.put(clazz, ormClassTool);
    }

    @Override
    public boolean containsTool(final Class<?> clazz) {
        return classToolMap.containsKey(clazz);
    }

    @SuppressWarnings("unchecked")
    @Override
    public <T> OrmClassTool<T> getOrmClassTool(final Class<T> clazz) throws OrmException {
        OrmClassTool<?> ormClazzTool = classToolMap.get(clazz);
        if (ormClazzTool==null) {
            jpOrm.register(clazz);
            ormClazzTool = classToolMap.get(clazz);
        }
        return (OrmClassTool<T>) ormClazzTool;
    }

    @Override
    public TypeFactory getTypeFactory() {
        return typeFactory;
    }

    /**
     * @return the dbProfile
     */
    @Override
    public DBProfile getDbProfile() {
        return dbProfile;
    }

    /**
     * @param dbProfile the dbProfile to set
     */
    public void setDbProfile(final DBProfile dbProfile) {
        this.dbProfile = dbProfile;
    }

    /**
     * @return the validatorService
     */
    @Override
    public ValidatorService getValidatorService() {
        return validatorService;
    }

    /**
     * @param validatorService the validatorService to set
     */
    public void setValidatorService(final ValidatorService validatorService) {
        this.validatorService = validatorService;
    }

    /**
     * @return the cacheManager
     */
    @Override
    public CacheManager getCacheManager() {
        return cacheManager;
    }

    /**
     * @param cacheManager the cacheManager to set
     */
    public void setCacheManager(final CacheManager cacheManager) {
        this.cacheManager = cacheManager;
    }

    @Override
    public PropertiesFactory getPropertiesFactory() {
        return propertiesFactory;
    }

    /**
     * @return the cacheStrategy
     */
    @Override
    public CacheStrategy getCacheStrategy() {
        return cacheStrategy;
    }

}
