/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Feb 13, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.mapper.relation;

import com.jpattern.orm.annotation.cascade.CascadeInfo;
import com.jpattern.orm.mapper.clazz.ClassField;
import com.jpattern.orm.persistor.reflection.GetManipulator;
import com.jpattern.orm.persistor.reflection.SetManipulator;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Feb 13, 2013
 *
 * @author Francesco Cina'
 * @version $Revision
 */
public class RelationOuterFKImpl<BEAN, P, MANIPULATOR> implements RelationOuterFK<BEAN, P, MANIPULATOR> {

    private final Class<P> relationWithClass;
    private final String javaFieldName;
    private GetManipulator<BEAN, MANIPULATOR> getManipulator;
    private SetManipulator<BEAN, MANIPULATOR> setManipulator;
    private final ClassField<P, Object> relationClassField;
    private final boolean oneToMany;
    private final CascadeInfo cascadeInfo;

    public RelationOuterFKImpl(final Class<P> relationWithClass, final ClassField<P, Object> relationClassField, final String javaFieldName, final boolean oneToMany, final CascadeInfo cascadeInfo) {
        this.relationWithClass = relationWithClass;
        this.relationClassField = relationClassField;
        this.javaFieldName = javaFieldName;
        this.oneToMany = oneToMany;
        this.cascadeInfo = cascadeInfo;
    }

    @Override
    public Class<P> getRelationWithClass() {
        return relationWithClass;
    }

    @Override
    public String getJavaFieldName() {
        return javaFieldName;
    }

    @Override
    public GetManipulator<BEAN, MANIPULATOR> getGetManipulator() {
        return getManipulator;
    }

    public void setGetManipulator(final GetManipulator<BEAN, MANIPULATOR> getManipulator) {
        this.getManipulator = getManipulator;
    }

    @Override
    public SetManipulator<BEAN, MANIPULATOR> getSetManipulator() {
        return setManipulator;
    }

    public void setSetManipulator(final SetManipulator<BEAN, MANIPULATOR> setManipulator) {
        this.setManipulator = setManipulator;
    }

    @Override
    public void copyFromTo(final BEAN source, final BEAN destination) {
        getSetManipulator().setValue(destination,  getGetManipulator().getValue(source) );
    }

    @Override
    public ClassField<P, Object> getRelationClassField() {
        return relationClassField;
    }

    @Override
    public boolean isOneToMany() {
        return oneToMany;
    }

    /**
     * @return the cascadeInfo
     */
    @Override
    public CascadeInfo getCascadeInfo() {
        return cascadeInfo;
    }

}
