package com.jpattern.orm.query.clause;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.query.SmartRenderableSqlSubElement;
import com.jpattern.orm.query.namesolver.NameSolver;
import com.jpattern.orm.util.ObjectBuilder;
import com.jpattern.orm.util.StringUtil;

/**
 * 
 * @author Francesco Cina
 *
 * 07/lug/2011
 */
public abstract class OrmCustomSelect<T extends Select<?>> extends SmartRenderableSqlSubElement implements Select<T> {

    public static String SQL_SELECT_SPLIT_PATTERN = "[^,]*[\\(][^\\)]*[\\)][^,]*|[^,]+"; //$NON-NLS-1$

    public static String SQL_SELECT_PROPERTY_PATTERN = "[a-zA-Z_0-9]+[\\.][a-zA-Z_0-9]+[\\.][a-zA-Z_0-9]+|[a-zA-Z_0-9]+[\\.][a-zA-Z_0-9]+"; //$NON-NLS-1$

    //    private static Pattern patternSelectClause = Pattern.compile(SQL_SELECT_SPLIT_PATTERN);
    private static Pattern patternProperty = Pattern.compile(SQL_SELECT_PROPERTY_PATTERN);

    private int versionStatus = 0;
    private boolean distinct = false;
    private final String[] selectFields;

    private List<String> _ignoredFields = ObjectBuilder.EMPTY_STRING_LIST;

    public OrmCustomSelect(final String[] selectFields) {
        this.selectFields = selectFields;
    }

    @Override
    public final void doElementRender(final StringBuilder queryBuilder, final NameSolver nameSolver) {
        queryBuilder.append("SELECT "); //$NON-NLS-1$
        if (this.distinct) {
            queryBuilder.append("DISTINCT "); //$NON-NLS-1$
        }

        int size = selectFields.length;
        boolean first = true;
        for (int i=0; i<size; i++) {
            String field = selectFields[i];
            if (!_ignoredFields.contains(field)) {

                if (!first) {
                    queryBuilder.append(", "); //$NON-NLS-1$
                } else {
                    first = false;
                }
                solveField(field.trim(), queryBuilder, nameSolver);

                //                final Matcher m = patternSelectClause.matcher(field);
                //                boolean loop = m.find();
                //                while (loop) {
                //                    solveField(m.group().trim(), queryBuilder, nameSolver);
                //                    loop = m.find();
                //                    if (loop) {
                //                        queryBuilder.append(", "); //$NON-NLS-1$
                //                    }
                //                }
            }
        }

        queryBuilder.append(" "); //$NON-NLS-1$
    }

    /**
     * @param string
     * @return
     */
    private void solveField(final String field, final StringBuilder queryBuilder, final NameSolver nameSolver) {
        if ( field.contains(",") || field.contains("(") || StringUtil.containsIgnoreCase(field, " as ") ) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            solveProperties( queryBuilder, field, nameSolver ) ;
        } else {
            queryBuilder.append( nameSolver.solvePropertyName( field ) );
            queryBuilder.append( " AS \"" ); //$NON-NLS-1$
            queryBuilder.append( field );
            queryBuilder.append( "\"" ); //$NON-NLS-1$
        }
    }

    @Override
    public final int getElementStatusVersion() {
        return this.versionStatus;
    }

    @Override
    public final void appendElementValues(final List<Object> values) {
        // do nothing
    }

    private void solveProperties(final StringBuilder queryBuilder, final String token, final NameSolver nameSolver) {
        final Matcher m = patternProperty.matcher(token);
        int beginIndex = 0;

        while (m.find()) {
            queryBuilder.append( token.substring(beginIndex, m.start()) );
            queryBuilder.append( nameSolver.solvePropertyName( m.group() ) );
            beginIndex = m.end();
        }

        queryBuilder.append( token.substring(beginIndex, token.length()) );
    }

    public T setDistinct(final boolean distinct) {
        this.distinct = distinct;
        this.versionStatus++;
        return select();
    }

    public boolean isDistinct() throws OrmException {
        return this.distinct;
    }

    /**
     * @param _ignoredFields
     */
    @Override
    public void ignore(final List<String> ignoredFields) {
        this._ignoredFields = ignoredFields;
    }

    protected abstract T select();

}
