package com.jpattern.orm.query.clause;

import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.query.SmartRenderableSqlSubElement;
import com.jpattern.orm.query.clause.from.FromElement;
import com.jpattern.orm.query.clause.from.FullOuterJoinElement;
import com.jpattern.orm.query.clause.from.InnerJoinElement;
import com.jpattern.orm.query.clause.from.JoinElement;
import com.jpattern.orm.query.clause.from.LeftOuterJoinElement;
import com.jpattern.orm.query.clause.from.NaturalJoinElement;
import com.jpattern.orm.query.clause.from.RightOuterJoinElement;
import com.jpattern.orm.query.namesolver.NameSolver;

/**
 * 
 * @author Francesco Cina
 *
 * 27/giu/2011
 */
public abstract class OrmFrom<T extends From<?>> extends SmartRenderableSqlSubElement implements From<T> {

    private final List<FromElement> joinElements = new ArrayList<FromElement>();
    private final ServiceCatalog serviceCatalog;
    private final Class<?> clazz;
    private final Integer mainNameSolverClassId;
    private NameSolver nameSolver;

    public OrmFrom (final ServiceCatalog serviceCatalog, final Class<?> clazz, final Integer nameSolverClassId, final NameSolver nameSolver) {
        this.serviceCatalog = serviceCatalog;
        this.clazz = clazz;
        this.mainNameSolverClassId = nameSolverClassId;
        this.nameSolver = nameSolver;
    }

    protected abstract T from();

    @Override
    public T join(final Class<?> joinClass) {
        return join(joinClass, joinClass.getSimpleName());
    }

    @Override
    public T join(final Class<?> joinClass, final String joinClassAlias) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new JoinElement(this.serviceCatalog, joinClass, nameSolverClassId) );
    }

    @Override
    public T naturalJoin(final Class<?> joinClass) {
        return naturalJoin(joinClass, joinClass.getSimpleName());
    }

    @Override
    public T naturalJoin(final Class<?> joinClass, final String joinClassAlias) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new NaturalJoinElement(this.serviceCatalog, joinClass, nameSolverClassId) );
    }

    @Override
    public T innerJoin(final Class<?> joinClass) {
        return innerJoin(joinClass, joinClass.getSimpleName());
    }

    @Override
    public T innerJoin(final Class<?> joinClass, final String joinClassAlias) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new InnerJoinElement(this.serviceCatalog, joinClass, nameSolverClassId) );
    }

    @Override
    public T innerJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
        return innerJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
    }

    @Override
    public T innerJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new InnerJoinElement(this.serviceCatalog, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty) );
    }

    @Override
    public T leftOuterJoin(final Class<?> joinClass) {
        return leftOuterJoin(joinClass, joinClass.getSimpleName());
    }

    @Override
    public T leftOuterJoin(final Class<?> joinClass, final String joinClassAlias) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new LeftOuterJoinElement(this.serviceCatalog, joinClass, nameSolverClassId) );
    }

    @Override
    public T leftOuterJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
        return leftOuterJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
    }

    @Override
    public T leftOuterJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new LeftOuterJoinElement(this.serviceCatalog, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty) );
    }

    @Override
    public T rightOuterJoin(final Class<?> joinClass) {
        return rightOuterJoin(joinClass, joinClass.getSimpleName());
    }

    @Override
    public T rightOuterJoin(final Class<?> joinClass, final String joinClassAlias) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new RightOuterJoinElement(this.serviceCatalog, joinClass, nameSolverClassId) );
    }

    @Override
    public T rightOuterJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
        return rightOuterJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
    }

    @Override
    public T rightOuterJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new RightOuterJoinElement(this.serviceCatalog, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty) );
    }

    @Override
    public T fullOuterJoin(final Class<?> joinClass) {
        return fullOuterJoin(joinClass, joinClass.getSimpleName());
    }

    @Override
    public T fullOuterJoin(final Class<?> joinClass, final String joinClassAlias) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new FullOuterJoinElement(this.serviceCatalog, joinClass, nameSolverClassId) );
    }

    @Override
    public T fullOuterJoin(final Class<?> joinClass, final String onLeftProperty, final String onRigthProperty) {
        return fullOuterJoin(joinClass, joinClass.getSimpleName(), onLeftProperty, onRigthProperty);
    }

    @Override
    public T fullOuterJoin(final Class<?> joinClass, final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        Integer nameSolverClassId = nameSolver.register(joinClass, joinClassAlias);
        return addJoinElement( new FullOuterJoinElement(this.serviceCatalog, joinClass, nameSolverClassId, onLeftProperty, onRigthProperty) );
    }

    @Override
    public final int getElementStatusVersion() {
        return this.joinElements.size();
    }

    @Override
    public final void doElementRender(final StringBuilder queryBuilder, final NameSolver localNameSolver) {
        final String alias = localNameSolver.normalizedAlias(this.mainNameSolverClassId);
        queryBuilder.append("FROM "); //$NON-NLS-1$
        queryBuilder.append(this.serviceCatalog.getOrmClassTool(this.clazz).getClassMap().getTableInfo().getTableNameWithSchema() );
        queryBuilder.append( " " ); //$NON-NLS-1$
        queryBuilder.append(alias);
        queryBuilder.append(" "); //$NON-NLS-1$
        for (final FromElement joinElement : this.joinElements) {
            joinElement.renderSqlElement(queryBuilder, localNameSolver);
        }
    }

    @Override
    public final void appendElementValues(final List<Object> values) {
        // do nothing
    }

    private T addJoinElement(final FromElement joinElement) {
        this.joinElements.add(joinElement);
        return from();
    }

}
