package com.jpattern.orm.query.clause.from;

import java.util.List;

import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.query.SmartRenderableSqlSubElement;
import com.jpattern.orm.query.namesolver.NameSolver;

/**
 * 
 * @author Francesco Cina
 *
 * 27/giu/2011
 */
public abstract class AFromElement extends SmartRenderableSqlSubElement implements FromElement {

    protected final Class<?> joinClass;
    protected final ServiceCatalog serviceCatalog;
    private final Integer nameSolverClassId;

    public AFromElement(final ServiceCatalog serviceCatalog, final Class<?> joinClass, final Integer nameSolverClassId) {
        this.serviceCatalog = serviceCatalog;
        this.joinClass = joinClass;
        this.nameSolverClassId = nameSolverClassId;
    }

    @Override
    public final int getElementStatusVersion() {
        return 0;
    }

    @Override
    public final void doElementRender(final StringBuilder queryBuilder, final NameSolver nameSolver) {
        String alias = nameSolver.normalizedAlias(getNameSolverClassId());
        queryBuilder.append( getJoinName() );
        queryBuilder.append(serviceCatalog.getOrmClassTool(joinClass).getClassMap().getTableInfo().getTableNameWithSchema() );
        queryBuilder.append( " " ); //$NON-NLS-1$
        queryBuilder.append(alias);

        if (hasOnClause()) {
            queryBuilder.append( " ON " ); //$NON-NLS-1$
            queryBuilder.append( nameSolver.solvePropertyName(onLeftProperty()) );
            queryBuilder.append( " = " ); //$NON-NLS-1$
            queryBuilder.append( nameSolver.solvePropertyName(onRightProperty()) );
        }

        queryBuilder.append( " " ); //$NON-NLS-1$
    }

    protected abstract String getJoinName();

    protected abstract boolean hasOnClause();

    protected abstract String onLeftProperty();

    protected abstract String onRightProperty();

    public Integer getNameSolverClassId() {
        return nameSolverClassId;
    }

    @Override
    public final void appendElementValues(final List<Object> values) {
        // do nothing
    }

}