package com.jpattern.orm.query.clause.where;

import java.util.List;

import com.jpattern.orm.exception.OrmQueryFormatException;
import com.jpattern.orm.query.SmartRenderableSqlSubElement;
import com.jpattern.orm.query.namesolver.NameSolver;

/**
 * 
 * @author Francesco Cina
 *
 * 19/giu/2011
 */
public abstract class APropertiesExpressionElement extends SmartRenderableSqlSubElement implements ExpressionElement {

    private final String firstProperty;
    private final String secondProperty;

    public APropertiesExpressionElement(final String firstProperty, final String secondProperty) {
        this.firstProperty = firstProperty;
        this.secondProperty = secondProperty;
    }

    private PropertyDecorator propertyDecorator = new NullPropertyDecorator();
    private PropertyDecorator valueDecorator = new NullPropertyDecorator();
    private int elementStatusVersion;

    protected PropertyDecorator getPropertyDecorator() {
        return propertyDecorator;
    }

    protected void setPropertyDecorator(final PropertyDecorator propertyDecorator) {
        elementStatusVersion++;
        this.propertyDecorator = propertyDecorator;
    }

    protected PropertyDecorator getValueDecorator() {
        return valueDecorator;
    }

    protected void setValueDecorator(final PropertyDecorator valueDecorator) {
        elementStatusVersion++;
        this.valueDecorator = valueDecorator;
    }

    @Override
    public final void doElementRender(final StringBuilder query, final NameSolver nameSolver) throws OrmQueryFormatException {
        getPropertyDecorator().decore( nameSolver.solvePropertyName(firstProperty) , query );
        query.append( " " ); //$NON-NLS-1$
        query.append( getExpressionElementKey() );
        query.append( " " ); //$NON-NLS-1$
        getPropertyDecorator().decore( nameSolver.solvePropertyName(secondProperty), query );
        query.append( " " ); //$NON-NLS-1$
    }

    public abstract String getExpressionElementKey();


    @Override
    public final void appendElementValues(final List<Object> values) {
        // do nothing
    }

    @Override
    public final int getElementStatusVersion() {
        return elementStatusVersion;
    }
}
