/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Feb 22, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.query.crud;

import com.jpattern.orm.exception.OrmNotUniqueResultException;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Feb 22, 2013
 *
 * @author Francesco Cina'
 * @version $Revision
 */
public interface Find<BEAN> {

    /**
     * Fetch the bean
     * @return
     */
    BEAN get();

    /**
     * Fetch the bean. An {@link OrmNotUniqueResultException} is thrown if the result is not unique.
     * @return
     */
    BEAN getUnique();

    /**
     * Return whether a bean exists with the specified id(s)
     * @return
     */
    boolean exist();

    /**
     * If true the bean's relations will not be fetched.
     * Default is false.
     * @param lazy
     * @return
     */
    Find<BEAN> lazy(boolean lazy);

    /**
     * Activate the cache for this query.
     * @param cacheName the of the cache to use
     * @return
     */
    Find<BEAN> cache(String cacheName);

    /**
     * The value of the Bean fields listed will not be fetched from the DB. This is useful to load only a partial Bean
     * to reduce the amount of work of the DB. Normally this is used to avoid loading LOB values when not needed.
     * @param fields
     * @return
     */
    Find<BEAN> ignore(String... fields);

    /**
     * The value of the Bean fields listed will not be fetched from the DB. This is useful to load only a partial Bean
     * to reduce the amount of work of the DB. Normally this is used to avoid loading LOB values when not needed.
     * If 'ignoreFieldsCondition' is false the fields will not be ignored fetched.
     * @param fields
     * @param ignoreFieldsCondition
     * @return
     */
    Find<BEAN> ignore(boolean ignoreFieldsCondition, String... fields);

}
