package com.jpattern.orm.query.find;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.query.LockMode;
import com.jpattern.orm.query.SmartRenderableSqlQuery;
import com.jpattern.orm.query.namesolver.NameSolver;
import com.jpattern.orm.query.namesolver.NameSolverImpl;
import com.jpattern.orm.session.ResultSetReader;
import com.jpattern.orm.session.ResultSetRowReader;
import com.jpattern.orm.session.SessionImpl;
import com.jpattern.orm.session.SqlPerformer;

/**
 * 
 * @author Francesco Cina
 *
 * 20/giu/2011
 */
public class CustomFindQueryOrm extends SmartRenderableSqlQuery implements CustomFindQuery {

    private final CustomFindSelectImpl select;
    private final SessionImpl session;
    private int _queryTimeout = 0;
    private int _maxRows = 0;
    private LockMode _lockMode = LockMode.NO_LOCK;
    private final CustomFindWhereImpl where = new CustomFindWhereImpl(this);
    private final CustomFindOrderByImpl orderBy = new CustomFindOrderByImpl(this);
    private final CustomFindFromImpl from;
    private int versionStatus = 0;
    private final NameSolver nameSolver;

    public CustomFindQueryOrm(final String[] selectFields, final ServiceCatalog serviceCatalog, final SessionImpl session, final Class<?> clazz, final String alias) {
        this.session = session;
        nameSolver = new NameSolverImpl(serviceCatalog, false);
        from = new CustomFindFromImpl(this, serviceCatalog, clazz, nameSolver.register(clazz, alias), nameSolver);
        select = new CustomFindSelectImpl(selectFields);
    }

    @Override
    public List<Object[]> getList() {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForList(renderSql(), values);
    }


    @Override
    public Object[] getUnique() throws OrmNotUniqueResultException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForArray(renderSql(), values);
    }

    @Override
    public final int getStatusVersion() {
        return versionStatus + select.getElementStatusVersion() + from.getElementStatusVersion() + where.getElementStatusVersion() + orderBy.getElementStatusVersion();
    }

    @Override
    public final void doRender(final StringBuilder queryBuilder) {
        select.renderSqlElement(queryBuilder, nameSolver);
        from.renderSqlElement(queryBuilder, nameSolver);
        where.renderSqlElement(queryBuilder, nameSolver);
        orderBy.renderSqlElement(queryBuilder, nameSolver);
        queryBuilder.append(_lockMode.getMode());
    }

    @Override
    public <T> T get(final ResultSetReader<T> rse) throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.query(renderSql(), rse, values);
    }

    @Override
    public <T> List<T> get(final ResultSetRowReader<T> rsrr) throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.query(renderSql(), rsrr, values);
    }

    @Override
    public <T> T getUnique(final ResultSetRowReader<T> rsrr) throws OrmException, OrmNotUniqueResultException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForUnique(renderSql(), rsrr, values);
    }

    @Override
    public int getInt() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForInt(renderSql(), values);
    }

    @Override
    public long getLong() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForLong(renderSql(), values);
    }

    @Override
    public double getDouble() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForDouble(renderSql(), values);
    }

    @Override
    public float getFloat() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForFloat(renderSql(), values);
    }

    @Override
    public String getString() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForString(renderSql(), values);
    }

    @Override
    public boolean getBoolean() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForBoolean(renderSql(), values);
    }

    @Override
    public BigDecimal getBigDecimal() throws OrmException {
        final List<Object> values = new ArrayList<Object>();
        where.appendElementValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setMaxRows(getMaxRows());
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.queryForBigDecimal(renderSql(), values);
    }

    public final int getMaxRows() throws OrmException {
        return _maxRows;
    }

    @Override
    public final CustomFindQuery queryTimeout(final int queryTimeout) {
        _queryTimeout = queryTimeout;
        return this;
    }

    public final int getQueryTimeout() {
        return _queryTimeout;
    }

    @Override
    public final CustomFindQuery maxRows(final int maxRows) throws OrmException {
        _maxRows = maxRows;
        return this;
    }

    @Override
    public CustomFindQuery lockMode(final LockMode lockMode) {
        _lockMode = lockMode;
        versionStatus++;
        return this;
    }

    public LockMode getLockMode() {
        return _lockMode;
    }

    @Override
    public final CustomFindWhere where() throws OrmException {
        return where;
    }

    @Override
    public final CustomFindOrderBy orderBy() throws OrmException {
        return orderBy;
    }

    @Override
    public final void appendValues(final List<Object> values) {
        where.appendElementValues(values);
    }

    @Override
    public CustomFindQuery join(final Class<?> joinClass) {
        return from.join(joinClass);
    }

    @Override
    public CustomFindQuery join(final Class<?> joinClass, final String joinClassAlias) {
        return from.join(joinClass, joinClassAlias);
    }

    @Override
    public CustomFindQuery naturalJoin(final Class<?> joinClass) {
        return from.naturalJoin(joinClass);
    }

    @Override
    public CustomFindQuery naturalJoin(final Class<?> joinClass, final String joinClassAlias) {
        return from.naturalJoin(joinClass, joinClassAlias);
    }

    @Override
    public CustomFindQuery innerJoin(final Class<?> joinClass) {
        return from.innerJoin(joinClass);
    }

    @Override
    public CustomFindQuery innerJoin(final Class<?> joinClass, final String joinClassAlias) {
        return from.innerJoin(joinClass, joinClassAlias);
    }

    @Override
    public CustomFindQuery innerJoin(final Class<?> joinClass, final String onLeftProperty,
            final String onRigthProperty) {
        return from.innerJoin(joinClass, onLeftProperty, onRigthProperty);
    }

    @Override
    public CustomFindQuery innerJoin(final Class<?> joinClass, final String joinClassAlias,
            final String onLeftProperty, final String onRigthProperty) {
        return from.innerJoin(joinClass, joinClassAlias, onLeftProperty,
                onRigthProperty);
    }

    @Override
    public CustomFindQuery leftOuterJoin(final Class<?> joinClass) {
        return from.leftOuterJoin(joinClass);
    }

    @Override
    public CustomFindQuery leftOuterJoin(final Class<?> joinClass,
            final String joinClassAlias) {
        return from.leftOuterJoin(joinClass, joinClassAlias);
    }

    @Override
    public CustomFindQuery leftOuterJoin(final Class<?> joinClass,
            final String onLeftProperty, final String onRigthProperty) {
        return from.leftOuterJoin(joinClass, onLeftProperty, onRigthProperty);
    }

    @Override
    public CustomFindQuery leftOuterJoin(final Class<?> joinClass,
            final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        return from.leftOuterJoin(joinClass, joinClassAlias, onLeftProperty,
                onRigthProperty);
    }

    @Override
    public CustomFindQuery rightOuterJoin(final Class<?> joinClass) {
        return from.rightOuterJoin(joinClass);
    }

    @Override
    public CustomFindQuery rightOuterJoin(final Class<?> joinClass,
            final String joinClassAlias) {
        return from.rightOuterJoin(joinClass, joinClassAlias);
    }

    @Override
    public CustomFindQuery rightOuterJoin(final Class<?> joinClass,
            final String onLeftProperty, final String onRigthProperty) {
        return from.rightOuterJoin(joinClass, onLeftProperty, onRigthProperty);
    }

    @Override
    public CustomFindQuery rightOuterJoin(final Class<?> joinClass,
            final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        return from.rightOuterJoin(joinClass, joinClassAlias, onLeftProperty,
                onRigthProperty);
    }

    @Override
    public CustomFindQuery fullOuterJoin(final Class<?> joinClass) {
        return from.fullOuterJoin(joinClass);
    }

    @Override
    public CustomFindQuery fullOuterJoin(final Class<?> joinClass,
            final String joinClassAlias) {
        return from.fullOuterJoin(joinClass, joinClassAlias);
    }

    @Override
    public CustomFindQuery fullOuterJoin(final Class<?> joinClass,
            final String onLeftProperty, final String onRigthProperty) {
        return from.fullOuterJoin(joinClass, onLeftProperty, onRigthProperty);
    }

    @Override
    public CustomFindQuery fullOuterJoin(final Class<?> joinClass,
            final String joinClassAlias, final String onLeftProperty, final String onRigthProperty) {
        return from.fullOuterJoin(joinClass, joinClassAlias, onLeftProperty,
                onRigthProperty);
    }

    @Override
    public String toString() {
        return from.toString();
    }

    @Override
    public CustomFindQuery distinct(final boolean distinct) {
        select.setDistinct(distinct);
        return this;
    }

    public boolean isDistinct() throws OrmException {
        return select.isDistinct();
    }

}
