package com.jpattern.orm.query.find;

import java.sql.ResultSet;
import java.util.List;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.query.BaseFindQuery;
import com.jpattern.orm.query.LockMode;
import com.jpattern.orm.query.OrmRowMapper;

/**
 * 
 * @author Francesco Cina
 *
 * 18/giu/2011
 */
public interface FindQueryCommon<BEAN> extends BaseFindQuery {

    /**
     * Execute the query returning the list of beans.
     * @return
     */
    List<BEAN> getList() throws OrmException;

    /**
     * Return the count of entities this query should return.
     * @return
     */
    int getRowCount() throws OrmException;

    /**
     * Execute the query returning either a single bean or an Exception.
     * @return
     * @throws OrmNotUniqueResultException if zero or more than one row are returned from the query
     */
    BEAN getUnique() throws OrmException, OrmNotUniqueResultException;

    /**
     * Execute the query returning either a single bean, if the {@link ResultSet} contains at least one row,
     * or null, if no matching beans are found).
     * @return
     * @throws OrmNotUniqueResultException if zero or more than one row are returned from the query
     */
    BEAN get() throws OrmException, OrmNotUniqueResultException;

    /**
     * Execute the query and for every bean returned calls the IOrmSerialResultReader.
     * The Beans are created one after the other to limit the memory usage.
     * @param srr
     * @throws OrmException
     */
    void get(OrmRowMapper<BEAN> srr) throws OrmException;

    /**
     * Return the sql used to calculate the row count of the execution of this query.
     * @return
     */
    String renderRowCountSql() throws OrmException;

    /**
     * Whether to use Distinct in the select clause
     * @return
     */
    FindQuery<BEAN> distinct(boolean distinct) throws OrmException;

    /**
     * Set the {@link LockMode} for the query
     * @param lockMode
     * @return
     */
    FindQuery<BEAN> lockMode(LockMode lockMode);

    /**
     * Set the maximum number of rows to return.
     * @param maxRows
     * @return
     */
    FindQuery<BEAN> maxRows(int maxRows) throws OrmException;

    /**
     * Set the query timeout.
     */
    FindQuery<BEAN> queryTimeout(int queryTimeout);

}
