package com.jpattern.orm.query.find.cache;

import java.util.List;

/**
 * 
 * @author ufo
 *
 */
public class CacheKey {

    private final String sql;
    private final List<Object> args;
    private final int maxRows;
    private final List<String> ignoredFields;

    public CacheKey(final String sql, final List<Object> args, final List<String> ignoredFields, final int maxRows) {
        this.sql = sql;
        this.args = args;
        this.ignoredFields = ignoredFields;
        this.maxRows = maxRows;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = (prime * result) + ((args == null) ? 0 : args.hashCode());
        result = (prime * result)
                + ((ignoredFields == null) ? 0 : ignoredFields.hashCode());
        result = (prime * result) + maxRows;
        result = (prime * result) + ((sql == null) ? 0 : sql.hashCode());
        return result;
    }

    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        CacheKey other = (CacheKey) obj;
        if (args == null) {
            if (other.args != null) {
                return false;
            }
        } else if (!args.equals(other.args)) {
            return false;
        }
        if (ignoredFields == null) {
            if (other.ignoredFields != null) {
                return false;
            }
        } else if (!ignoredFields.equals(other.ignoredFields)) {
            return false;
        }
        if (maxRows != other.maxRows) {
            return false;
        }
        if (sql == null) {
            if (other.sql != null) {
                return false;
            }
        } else if (!sql.equals(other.sql)) {
            return false;
        }
        return true;
    }

}
