/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Mar 5, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.query.find.cache;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jpattern.orm.cache.Cache;
import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.query.OrmRowMapper;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 5, 2013
 *
 * @author Francesco Cina'
 * @version $Revision
 */
public class CacheStrategyImpl implements CacheStrategy {

    private Logger logger = LoggerFactory.getLogger(getClass());

    private final CacheStrategyEntry<Object> nullCallback = new CacheStrategyEntry<Object>() {
        @Override
        public void add(final Object bean) {//do nothing
        }
        @Override
        public void end() {//do nothing
        }
    };

    private ServiceCatalog serviceCatalog;

    public CacheStrategyImpl(final ServiceCatalog serviceCatalog) {
        this.serviceCatalog = serviceCatalog;
    }

    @Override
    @SuppressWarnings("unchecked")
    public <BEAN> void find(final String cacheName, final String sql, final List<Object> values,
            final List<String> ignoredFields, final int maxRows, final OrmRowMapper<BEAN> srr,
            final CacheStrategyCallback<BEAN> cacheStrategyCallback) {

        if ((cacheName==null) || cacheName.isEmpty()) {
            logger.trace("Cache disabled for query [{}]", sql); //$NON-NLS-1$
            cacheStrategyCallback.doWhenNotInCache((CacheStrategyEntry<BEAN>) nullCallback);
            return;
        }

        final Cache cache = serviceCatalog.getCacheManager().getCache(cacheName);
        final CacheKey key = new CacheKey(sql, values, ignoredFields, maxRows);
        if (logger.isDebugEnabled()) {
            logger.debug("Using cache [{}] for query [{}], maxRows [{}], ignoreFields {}, values {}", new Object[]{cacheName, sql, maxRows, ignoredFields, values}); //$NON-NLS-1$
            logger.debug("Query key hashcode [{}]", key.hashCode()); //$NON-NLS-1$
        }

        List<BEAN> cached = (List<BEAN>) cache.get(key);
        if (cached!=null) {
            logger.debug("Sql results found in cache"); //$NON-NLS-1$
            for (int i = 0; i < cached.size(); i++) {
                srr.read(cached.get(i), i);
            }
        } else {
            logger.debug("Sql results NOT found in cache"); //$NON-NLS-1$
            cacheStrategyCallback.doWhenNotInCache(new CacheStrategyEntry<BEAN>() {

                List<BEAN> beans = new ArrayList<BEAN>();

                @Override
                public void add(final BEAN bean) {
                    beans.add(bean);
                }

                @Override
                public void end() {
                    if(!beans.isEmpty()) {
                        cache.put(key, beans);
                    }
                }
            });
        }

    }

}
