/* ----------------------------------------------------------------------------
 *     PROJECT : EURES
 *
 *     PACKAGE : com.jpattern.orm.query.namesolver
 *        FILE : PropertiesFactory.java
 *
 *  CREATED BY : ARHS Developments
 *          ON : Mar 9, 2013
 *
 * MODIFIED BY : ARHS Developments
 *          ON : $LastChangedDate
 *     VERSION : $LastChangedRevision
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2011 European Commission - DG EMPL
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.query.namesolver;

import java.util.Map;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.query.Property;
import com.jpattern.orm.util.ObjectBuilder;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 9, 2013
 *
 * @author ARHS Developments - Francesco Cina
 * @version $Revision
 */
public class PropertiesFactory {

    private final Map<String, Property> PROPERTIES = ObjectBuilder.newMaxSizeMap(1000);

    public Property property(final String propertyValue) {
        Property property = PROPERTIES.get(propertyValue);
        if (property==null) {
            property = new Property(alias(propertyValue), field(propertyValue));
            PROPERTIES.put(propertyValue, property);
        }
        return property;
    }

    private String alias(final String property) throws OrmException {
        if (property.contains(".")) { //$NON-NLS-1$
            return property.substring(0, property.lastIndexOf(".")); //$NON-NLS-1$
        }
        return null;
    }

    private String field(final String property) throws OrmException {
        try {
            return property.substring(property.lastIndexOf(".")+1); //$NON-NLS-1$
        } catch (final Exception e) {
            throw new OrmException("Error parsing property [" + property + "], the format must be CLASS_NAME.CLASS_FIELD or CLASS_ALIAS.CLASS_FIELD" ); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

}
