package com.jpattern.orm.query.saveorupdate;

import java.util.Collection;
import java.util.List;

import com.jpattern.orm.mapper.OrmClassTool;
import com.jpattern.orm.mapper.clazz.ClassField;
import com.jpattern.orm.mapper.relation.RelationInnerFK;
import com.jpattern.orm.mapper.relation.RelationOuterFK;
import com.jpattern.orm.persistor.reflection.SetManipulator;
import com.jpattern.orm.session.SessionImpl;

/**
 * 
 * @author Francesco Cina
 *
 * 10/lug/2011
 */
public abstract class ASaveOrUpdateQueryOrm<BEAN> {

    @SuppressWarnings("unchecked")
    protected final <RELATION> void saveOrUpdateInnerRelations(final boolean cascade, final BEAN bean, final OrmClassTool<BEAN> ormClassTool, final SessionImpl session, final SaveOrUpdateType saveOrUpdateType) {
        if (cascade) {
            List<RelationInnerFK<BEAN, ?>> relations = ormClassTool.getClassMap().getInnerRelations();
            for (RelationInnerFK<BEAN, ?> relationInnerFk : relations) {
                RelationInnerFK<BEAN, RELATION> relationField = (RelationInnerFK<BEAN, RELATION>) relationInnerFk;
                RELATION innerBean = relationField.getGetManipulator().getValue(bean);
                RELATION saved = saveOrUpdateType.getStrategy().now(session, innerBean, cascade, relationInnerFk.getCascadeInfo());
                relationField.getSetManipulator().setValue(bean, saved);
            }
        }
    }

    protected final <RELATION> void saveOrUpdateOuterRelations(final boolean cascade, final BEAN bean, final OrmClassTool<BEAN> ormClassTool, final SessionImpl session, final SaveOrUpdateType saveOrUpdateType) {
        if (cascade) {
            List<RelationOuterFK<BEAN, ?, ?>> relations = ormClassTool.getClassMap().getOuterRelations();
            if (!relations.isEmpty()) {
                String[] pks = ormClassTool.getClassMap().getPrimaryKeyColumnJavaNames();
                Object beanPrimaryKeyValue = ormClassTool.getOrmPersistor().getPropertyValues(pks, bean)[0];
                for (RelationOuterFK<BEAN,? ,?> relation : relations) {

                    ClassField<RELATION, Object> relationClassField = (ClassField<RELATION, Object>) relation.getRelationClassField();
                    Object saved = null;
                    if (relation.isOneToMany()) {
                        Collection<RELATION> relatedBeans = (Collection<RELATION>) relation.getGetManipulator().getValue(bean);
                        for (RELATION relatedBean : relatedBeans) {
                            relationClassField.getSetManipulator().setValue(relatedBean, beanPrimaryKeyValue);
                        }
                        saved = saveOrUpdateType.getStrategy().now(session, relatedBeans, cascade, relation.getCascadeInfo());
                    } else {
                        RELATION relatedBean = (RELATION) relation.getGetManipulator().getValue(bean);
                        if (relatedBean!=null) {
                            relationClassField.getSetManipulator().setValue(relatedBean, beanPrimaryKeyValue);
                            saved = saveOrUpdateType.getStrategy().now(session, relatedBean, cascade, relation.getCascadeInfo());
                        }
                    }
                    (( SetManipulator<BEAN, Object>) relation.getSetManipulator()).setValue(bean, saved);
                }
            }
        }
    }
}
