/* ----------------------------------------------------------------------------
 *     PROJECT : EURES
 *
 *     PACKAGE : com.jpattern.orm.query.saveorupdate
 *        FILE : SaveOrUpdateType.java
 *
 *  CREATED BY : ARHS Developments
 *          ON : Mar 10, 2013
 *
 * MODIFIED BY : ARHS Developments
 *          ON : $LastChangedDate
 *     VERSION : $LastChangedRevision
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2011 European Commission - DG EMPL
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.query.saveorupdate;

import java.util.Collection;

import com.jpattern.orm.annotation.cascade.CascadeInfo;
import com.jpattern.orm.session.SessionImpl;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 10, 2013
 *
 * @author ARHS Developments - Francesco Cina
 * @version $Revision
 */
public enum SaveOrUpdateType {

    SAVE( new SaveOrUpdateStrategy() {
        @Override
        public <RELATION> RELATION now(final SessionImpl session, final RELATION innerBean, final boolean cascade, final CascadeInfo cascadeInfo) {
            if (cascadeInfo.onSave()) {
                return session.save(innerBean).cascade(cascade).now();
            }
            return innerBean;
        }
        @Override
        public <RELATION> Collection<RELATION> now(final SessionImpl session, final Collection<RELATION> innerBeans, final boolean cascade, final CascadeInfo cascadeInfo) {
            if (cascadeInfo.onSave()) {
                return session.save(innerBeans).cascade(cascade).now();
            }
            return innerBeans;
        }
    }),
    UPDATE( new SaveOrUpdateStrategy() {
        @Override
        public <RELATION> RELATION now(final SessionImpl session, final RELATION innerBean, final boolean cascade, final CascadeInfo cascadeInfo) {
            if (cascadeInfo.onSave()) {
                return session.update(innerBean).cascade(cascade).now();
            }
            return innerBean;
        }
        @Override
        public <RELATION> Collection<RELATION> now(final SessionImpl session, final Collection<RELATION> innerBeans, final boolean cascade, final CascadeInfo cascadeInfo) {
            if (cascadeInfo.onSave()) {
                return session.update(innerBeans).cascade(cascade).now();
            }
            return innerBeans;
        }
    }),
    SAVE_OR_UPDATE( new SaveOrUpdateStrategy() {
        @Override
        public <RELATION> RELATION now(final SessionImpl session, final RELATION innerBean, final boolean cascade, final CascadeInfo cascadeInfo) {
            return session.saveOrUpdate(innerBean, cascadeInfo).cascade(cascade).now();
        }
        @Override
        public <RELATION> Collection<RELATION> now(final SessionImpl session, final Collection<RELATION> innerBeans, final boolean cascade, final CascadeInfo cascadeInfo) {
            return session.saveOrUpdate(innerBeans, cascadeInfo).cascade(cascade).now();
        }
    });

    private final SaveOrUpdateStrategy strategy;

    SaveOrUpdateType(final SaveOrUpdateStrategy strategy) {
        this.strategy = strategy;
    }

    /**
     * @return the strategy
     */
    public SaveOrUpdateStrategy getStrategy() {
        return strategy;
    }

}
