package com.jpattern.orm.query.update;

import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.query.SmartRenderableSqlQuery;
import com.jpattern.orm.query.namesolver.NameSolver;
import com.jpattern.orm.query.namesolver.NameSolverImpl;
import com.jpattern.orm.session.SessionImpl;
import com.jpattern.orm.session.SqlPerformer;

/**
 * 
 * @author Francesco Cina
 *
 * 10/lug/2011
 */
public class CustomUpdateQueryImpl extends SmartRenderableSqlQuery implements CustomUpdateQuery {

    private final CustomUpdateSetImpl set = new CustomUpdateSetImpl(this);
    private final CustomUpdateWhereImpl where = new CustomUpdateWhereImpl(this);
    private final NameSolver nameSolver;
    private int _queryTimeout = 0;
    private final Class<?> clazz;
    private final ServiceCatalog serviceCatalog;
    private final SessionImpl session;

    public CustomUpdateQueryImpl(final Class<?> clazz, final ServiceCatalog serviceCatalog, final SessionImpl session) {
        this.clazz = clazz;
        this.serviceCatalog = serviceCatalog;
        this.session = session;
        nameSolver = new NameSolverImpl(serviceCatalog, true);
        nameSolver.register(clazz, clazz.getSimpleName());
    }

    @Override
    public CustomUpdateWhere where() {
        return where;
    }

    @Override
    public int now() {
        final List<Object> values = new ArrayList<Object>();
        appendValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setQueryTimeout(getQueryTimeout());
        return sqlExec.update(renderSql(), values);
    }

    @Override
    public final void appendValues(final List<Object> values) {
        set.appendElementValues(values);
        where.appendElementValues(values);
    }

    @Override
    public CustomUpdateQuery queryTimeout(final int queryTimeout) {
        _queryTimeout = queryTimeout;
        return this;
    }

    public int getQueryTimeout() {
        return _queryTimeout;
    }

    @Override
    public int getStatusVersion() {
        return set.getElementStatusVersion() + where.getElementStatusVersion();
    }

    @Override
    public final void doRender(final StringBuilder queryBuilder) {
        queryBuilder.append("UPDATE "); //$NON-NLS-1$
        queryBuilder.append(serviceCatalog.getOrmClassTool(clazz).getClassMap().getTableInfo().getTableNameWithSchema() );
        queryBuilder.append(" "); //$NON-NLS-1$
        set.renderSqlElement(queryBuilder, nameSolver);
        where.renderSqlElement(queryBuilder, nameSolver);
    }

    @Override
    public CustomUpdateSet set() {
        return set;
    }

}
