/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Feb 23, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.query.update;

import java.util.Map;
import java.util.Map.Entry;

import com.jpattern.orm.exception.OrmOptimisticLockException;
import com.jpattern.orm.exception.OrmReflectionException;
import com.jpattern.orm.mapper.OrmClassTool;
import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.persistor.OrmPersistor;
import com.jpattern.orm.query.find.FindWhere;
import com.jpattern.orm.query.saveorupdate.ASaveOrUpdateQueryOrm;
import com.jpattern.orm.query.saveorupdate.SaveOrUpdateType;
import com.jpattern.orm.session.SessionImpl;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Feb 23, 2013
 *
 * @author Francesco Cina'
 * @version $Revision
 */
public class UpdateQueryOrm<BEAN> extends ASaveOrUpdateQueryOrm<BEAN> implements UpdateQuery<BEAN> {

    private boolean _cascade;
    private SessionImpl session;
    private BEAN bean;
    private OrmClassTool<BEAN> ormClassTool;
    private int queryTimeout;
    private Class<BEAN> clazz;
    private SaveOrUpdateType _saveOrUpdateType = SaveOrUpdateType.UPDATE;

    /**
     * @param newBean
     * @param serviceCatalog
     * @param ormSession
     */
    public UpdateQueryOrm(final BEAN bean, final ServiceCatalog serviceCatalog, final SessionImpl session) {
        this.bean = bean;
        this.clazz= (Class<BEAN>) bean.getClass();
        this.ormClassTool = serviceCatalog.getOrmClassTool(clazz);
        this.session = session;
    }

    @Override
    public UpdateQuery<BEAN> value(final String propertyName, final Object value) {
        try {
            ormClassTool.getClassMap().getClassFieldByJavaName(propertyName).getSetManipulator().setValue(bean, value);
            return this;
        } catch (Exception e) {
            throw new OrmReflectionException(e);
        }
    }

    @Override
    public BEAN now() {
        saveOrUpdateInnerRelations(_cascade, bean, ormClassTool, session, _saveOrUpdateType);

        final OrmPersistor<BEAN> persistor = ormClassTool.getOrmPersistor();

        //CHECK IF OBJECT HAS A 'VERSION' FIELD
        if (persistor.isVersionable()) {
            FindWhere<BEAN> query = (FindWhere<BEAN>) session.findQuery(bean.getClass()).where();
            String[] keys = ormClassTool.getClassMap().getPrimaryKeyAndVersionColumnJavaNames();
            Object[] values = persistor.getPropertyValues(keys, bean);
            for (int i = 0; i < keys.length; i++) {
                query.eq(keys[i], values[i]);
            }
            if ( query.getRowCount()==0 ) {
                throw new OrmOptimisticLockException("The bean of class [" + bean.getClass() + "] cannot be updated. Version in the DB is not the expected one."); //$NON-NLS-1$ //$NON-NLS-2$
            }
            ormClassTool.getOrmPersistor().increaseVersion(bean, false);
        }

        CustomUpdateQuery updateQuery = session.updateQuery(bean.getClass()).queryTimeout(queryTimeout);

        CustomUpdateWhere updateQueryWhere = updateQuery.where();
        String[] pks = ormClassTool.getClassMap().getPrimaryKeyColumnJavaNames();
        Object[] pkValues = persistor.getPropertyValues(pks, bean);
        for (int i = 0; i < pks.length; i++) {
            updateQueryWhere.eq(pks[i], pkValues[i]);
        }

        CustomUpdateSet updateQuerySet = updateQuery.set();
        String[] notPks = ormClassTool.getClassMap().getNotPrimaryKeyColumnJavaNames();
        Object[] notPkValues = persistor.getPropertyValues(notPks, bean);
        for (int i = 0; i < notPks.length; i++) {
            updateQuerySet.eq(notPks[i], notPkValues[i]);
        }

        updateQuery.now();
        saveOrUpdateOuterRelations(_cascade, bean, ormClassTool, session, _saveOrUpdateType);
        return bean;
    }

    @Override
    public UpdateQuery<BEAN> setQueryTimeout(final int queryTimeout) {
        this.queryTimeout = queryTimeout;
        return this;
    }

    @Override
    public int getQueryTimeout() {
        return queryTimeout;
    }

    @Override
    public UpdateQuery<BEAN> values(final Map<String, Object> properties) {
        for (Entry<String, Object> propertyEntry : properties.entrySet()) {
            value(propertyEntry.getKey(), propertyEntry.getValue());
        }
        return this;
    }

    @Override
    public UpdateQuery<BEAN> cascade(final boolean cascade) {
        this._cascade = cascade;
        return this;
    }

    @Override
    public UpdateQuery<BEAN> saveOrUpdate(final SaveOrUpdateType saveOrUpdateType) {
        _saveOrUpdateType = saveOrUpdateType;
        return this;
    }

}
