package com.jpattern.orm.session;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jpattern.orm.dialect.DBType;
import com.jpattern.orm.dialect.DetermineDBType;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.transaction.Transaction;
import com.jpattern.orm.transaction.TransactionDefinition;

/**
 * 
 * @author Francesco Cina
 *
 * 21/mag/2011
 */
public abstract class SessionProvider {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private DBType dbType;

    public abstract DataSource getDataSource();

    public abstract Transaction getTransaction(TransactionDefinition transactionDefinition) throws OrmException;

    public abstract SqlPerformerStrategy sqlPerformerStrategy() throws OrmException;

    public final DBType getDBType() {
        if (dbType==null) {
            Connection connection = null;
            dbType = DBType.UNKNOWN;
            try
            {
                DataSource dataSource = getDataSource();
                if (dataSource!=null) {
                    connection = dataSource.getConnection();
                    DatabaseMetaData metaData = connection.getMetaData();

                    String driverName = metaData.getDriverName();
                    String driverVersion = metaData.getDriverVersion();
                    String url = metaData.getURL();
                    String databaseProductName = metaData.getDatabaseProductName();

                    getLogger().info("DB username: " + metaData.getUserName()); //$NON-NLS-1$
                    getLogger().info("DB driver name: " + driverName); //$NON-NLS-1$
                    getLogger().info("DB driver version: " + driverVersion); //$NON-NLS-1$
                    getLogger().info("DB url: " + url); //$NON-NLS-1$
                    getLogger().info("DB product name: " + databaseProductName); //$NON-NLS-1$
                    getLogger().info("DB product version: " + metaData.getDatabaseProductVersion()); //$NON-NLS-1$

                    dbType = new DetermineDBType().determineDBType(driverName, url, databaseProductName);
                }
            }
            catch (SQLException ex)
            {
                getLogger().warn("Error while determining the database type"); //$NON-NLS-1$
            }
            finally
            {
                if (connection != null)
                {
                    try
                    {
                        connection.close();
                    }
                    catch (SQLException ex)
                    {
                        // we ignore this one
                    }
                }
            }
            getLogger().info("DB type is " + dbType); //$NON-NLS-1$
        }
        return dbType;
    }

    public Logger getLogger() {
        return logger;
    }
}
