package com.jpattern.orm.session.datasource;

import javax.sql.DataSource;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.session.SqlPerformerStrategy;
import com.jpattern.orm.transaction.Transaction;
import com.jpattern.orm.transaction.TransactionDefinition;

/**
 * 
 * @author Francesco Cina
 *
 * 21/mag/2011
 */
public class DataSourceSessionProvider extends SessionProvider {

    private final ThreadLocal<DataSourceConnectionImpl> threadLocalConnection = new ThreadLocal<DataSourceConnectionImpl>();
    private final DataSourceTransactionManager dataSourceTransactionManager = new DataSourceTransactionManager();
    private final DataSource dataSource;
    private final SqlPerformerStrategy sqlPerformerStrategy  = new DataSourceSqlPerformerStrategy(this);
    private final DataSourceTransactionManager transactionManager = new DataSourceTransactionManager();

    public DataSourceSessionProvider(final DataSource dataSource) {
        this.dataSource = dataSource;
    }

    final DataSourceConnectionImpl getConnection(final boolean readOnly, final DataSourceConnectionCaller connectionCaller) {
        getLogger().debug("Connection asked by [{}]", connectionCaller ); //$NON-NLS-1$
        DataSourceConnectionImpl conn = threadLocalConnection.get();
        if ((conn==null) || !conn.isValid()) {
            getLogger().debug("No valid connections found, a new one will be created"); //$NON-NLS-1$
            conn = new DataSourceConnectionImpl(dataSource, readOnly);
            threadLocalConnection.set(conn);
        }
        conn.setReadOnly(readOnly);
        conn.addCaller(connectionCaller);
        return conn;
    }

    @Override
    public DataSource getDataSource() {
        return dataSource;
    }

    /**
     * @return the dataSourceTransactionManager
     */
    public DataSourceTransactionManager getDataSourceTransactionManager() {
        return dataSourceTransactionManager;
    }

    @Override
    public Transaction getTransaction(final TransactionDefinition transactionDefinition) throws OrmException {
        return transactionManager.startTransaction(this, transactionDefinition);
    }

    @Override
    public SqlPerformerStrategy sqlPerformerStrategy() throws OrmException {
        return sqlPerformerStrategy;
    }
}
