package com.jpattern.orm.session.datasource;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmRollbackException;
import com.jpattern.orm.transaction.Transaction;
import com.jpattern.orm.transaction.TransactionDefinition;
import com.jpattern.orm.transaction.TransactionIsolation;

/**
 * 
 * @author Francesco Cina
 *
 * 18/giu/2011
 */
public class DataSourceTransaction implements Transaction, DataSourceConnectionCaller {

    private final DataSourceConnection conn;
    private final DataSourceTransactionManager transactionManager;
    private boolean isClosed = false;
    private boolean rollbackOnly = false;

    public DataSourceTransaction(final DataSourceSessionProvider dataSourceSessionProvider, final TransactionDefinition transactionDefinition, final DataSourceTransactionManager dataSourceTransactionManager) {
        transactionManager = dataSourceTransactionManager;
        conn = dataSourceSessionProvider.getConnection(transactionDefinition.isReadOnly(), this);
        if (transactionDefinition.getIsolationLevel() != TransactionIsolation.DEFAULT) {
            getConnection().setTransactionIsolation(transactionDefinition.getIsolationLevel().getTransactionIsolation());
        }
    }

    @Override
    public void setRollbackOnly() throws OrmException {
        transactionManager.setRollbackOnly(this);
    }

    @Override
    public void rollback() throws OrmException {
        transactionManager.rollback(this);
    }

    @Override
    public void commit() throws OrmException, OrmRollbackException {
        transactionManager.commit(this);
    }


    /**
     * @return the conn
     */
    public DataSourceConnection getConnection() {
        return conn;
    }

    /**
     * @return the isClosed
     */
    @Override
    public boolean isClosed() {
        return isClosed;
    }

    /**
     * @param isClosed the isClosed to set
     */
    public void setClosed(final boolean isClosed) {
        this.isClosed = isClosed;
    }

    /**
     * @return the rollbackOnly
     */
    public boolean isRollbackOnly() {
        return rollbackOnly;
    }

    /**
     * @param rollbackOnly the rollbackOnly to set
     */
    public void setRollbackOnly(final boolean rollbackOnly) {
        this.rollbackOnly = rollbackOnly;
    }

}
