/* ----------------------------------------------------------------------------
 *     PROJECT : EURES
 *
 *     PACKAGE : com.jpattern.orm.session.datasource
 *        FILE : DataSourceTransactionManager.java
 *
 *  CREATED BY : ARHS Developments
 *          ON : Mar 9, 2013
 *
 * MODIFIED BY : ARHS Developments
 *          ON : $LastChangedDate
 *     VERSION : $LastChangedRevision
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2011 European Commission - DG EMPL
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.session.datasource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jpattern.orm.transaction.TransactionDefinition;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 9, 2013
 *
 * @author ARHS Developments - Francesco Cina
 * @version $Revision
 */
public class DataSourceTransactionManager {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    public DataSourceTransaction startTransaction(final DataSourceSessionProvider dataSourceSessionProvider, final TransactionDefinition transactionDefinition) {
        logger.debug("Starting new Transaction"); //$NON-NLS-1$
        return new DataSourceTransaction(dataSourceSessionProvider, transactionDefinition, this);
    }

    public void setRollbackOnly(final DataSourceTransaction transaction) {
        logger.debug("Set transaction as rollback only"); //$NON-NLS-1$
        transaction.setRollbackOnly(true);
        transaction.getConnection().setRollbackOnly();
    }

    public void commit(final DataSourceTransaction transaction) {
        if (transaction.isRollbackOnly()) {
            rollback(transaction);
            return;
        }
        try {
            if (!transaction.isClosed() && !transaction.getConnection().isClosed()) {
                logger.debug("Commit called"); //$NON-NLS-1$
                transaction.getConnection().commit();
            }
        } finally {
            transaction.getConnection().close(transaction);
            transaction.setClosed(true);
        }
    }

    public void rollback(final DataSourceTransaction transaction) {
        setRollbackOnly(transaction);
        try {
            if (!transaction.isClosed() && !transaction.getConnection().isClosed()) {
                logger.debug("Rollback called"); //$NON-NLS-1$
                transaction.getConnection().rollback();
            }
        } finally {
            transaction.getConnection().close(transaction);
            transaction.setClosed(true);
        }
    }

}
